package tk.losi.arduinocarremote;

import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.icu.util.Calendar;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import java.io.IOException;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;

public class MainActivity extends AppCompatActivity {
    private static final int REQUEST_ENABLE_BT = 0;

    private final List<String> devices = new ArrayList<>();       // List of bluetooth device names
    private final List<String> deviceAddr = new ArrayList<>();    // List of bluetooth device addresses

    private static boolean headlightsState = false;

    private static final long sendingInterval = 25;
    private long lastTime = System.currentTimeMillis();

    static final UUID myUUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    private BluetoothSocket btSocket = null;

    private TextView steerText;
    private TextView powerText;
    private SeekBar steer;
    private SeekBar power;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // Set application to Landscape Fullscreen
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);

        // Show device selection screen
        setContentView(R.layout.select_blueooth_device);

        // Get the bluetooth adapter and turn bluetooth on
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        if (!bluetoothAdapter.isEnabled()) {
            Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
        }

        // Get paired devices
        Set<BluetoothDevice> pairedDevices = bluetoothAdapter.getBondedDevices();

        if (pairedDevices.size() > 0) {
            // There are paired devices. Get the name and address of each paired device.
            for (BluetoothDevice device : pairedDevices) {
                String deviceName = device.getName();
                String deviceHardwareAddress = device.getAddress(); // MAC address
                devices.add(deviceName);
                deviceAddr.add(deviceHardwareAddress);
            }
        }

        // Setup UI List of devices
        ListView devicesList = findViewById(R.id.devicesListView);
        ArrayAdapter<String> devicesAdapter = new ArrayAdapter<>(
                this,
                android.R.layout.simple_list_item_1,
                devices
        );

        devicesList.setAdapter(devicesAdapter);

        // Setup connection to device on click
        devicesList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                try {
                    if (btSocket == null) {
                        // Connect the app with the selected device
                        BluetoothDevice hc = bluetoothAdapter.getRemoteDevice(deviceAddr.get(position));
                        btSocket = hc.createInsecureRfcommSocketToServiceRecord(myUUID);
                        BluetoothAdapter.getDefaultAdapter().cancelDiscovery();

                        btSocket.connect();
                    }
                }
                catch (IOException e) {
                    e.printStackTrace();
                }
                Toast.makeText(MainActivity.this, "Connected to " + devices.get(position), Toast.LENGTH_SHORT).show();
                // Change to remote control screen
                setContentView(R.layout.activity_main);
                FindViews();
            }
        });
    }

    // Find the sliders for power and steer
    public void FindViews() {
        steer = findViewById(R.id.steer);
        steer.setOnSeekBarChangeListener(steerListener);

        power = findViewById(R.id.power);
        power.setOnSeekBarChangeListener(powerListener);

        steerText = findViewById(R.id.steerText);
        powerText = findViewById(R.id.powerText);

        steerText.setText("Steer: 0");
        powerText.setText("Power: 0");
    }

    SeekBar.OnSeekBarChangeListener steerListener = new SeekBar.OnSeekBarChangeListener() {
        @Override
        public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
            // Send current steer to the car if enough time has passed
            steerText.setText("Steer: " + (progress - 50));
            try {
                if(Math.abs(System.currentTimeMillis() - lastTime) > sendingInterval) {
                    lastTime = System.currentTimeMillis();
                    btSocket.getOutputStream().write(("S" + String.format("%03d", progress)).getBytes());
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onStartTrackingTouch(SeekBar seekBar) {
        }

        @Override
        public void onStopTrackingTouch(SeekBar seekBar) {
            // Send Idle steer to the car
            seekBar.setProgress(50);
            try {
                btSocket.getOutputStream().write(("S" + String.format("%03d", 50)).getBytes());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    };

    SeekBar.OnSeekBarChangeListener powerListener = new SeekBar.OnSeekBarChangeListener() {
        @Override
        public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
            // Send current power to the car if enough time has passed
            powerText.setText("Power: " + (progress - 50));

            try {
                if(Math.abs(System.currentTimeMillis() - lastTime) > sendingInterval) {
                    lastTime = System.currentTimeMillis();
                    btSocket.getOutputStream().write(("P" + String.format("%03d", progress)).getBytes());
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onStartTrackingTouch(SeekBar seekBar) {
        }

        @Override
        public void onStopTrackingTouch(SeekBar seekBar) {
            seekBar.setProgress(50);
            // Send idle power to the car
            try {
                btSocket.getOutputStream().write(("P" + String.format("%03d", 50)).getBytes());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    };

    public void headLightsButtonClicked(View view) {
        Button button = findViewById(R.id.headlightsbtn);
        try {
                // Turn On Headlights Command
                String command = "NHDL";

                // If Headlights are on, turn them off
                if (headlightsState) {
                    command = "FHDL";
                    headlightsState = false;
                    button.setText("Turn On Headlights");
                } else {
                    headlightsState = true;
                    button.setText("Turn Off Headlights");
                }

                // Send command to the car
                btSocket.getOutputStream().write(command.getBytes());

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void resetButtonClicked(View view) {
        try {
            // Send reset command to the car
            btSocket.getOutputStream().write("RRRRRRRR".getBytes());

        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}