#include <SoftwareSerial.h>

// Motor 1 control pins
#define enA 3
#define in1 7
#define in2 6

// Motor 2 control pins
#define enB 5
#define in3 4
#define in4 2

// Bluetooth module comm pins
#define RxD 10
#define TxD 11


// Headlight pins
#define HEADLIGHT1 13
#define HEADLIGHT2 12

int motorASpeed = 0;
int motorBSpeed = 0;
int steer = 0;
int power = 0;

SoftwareSerial BTSerial(RxD, TxD);

// Set default motors state
void setMotorPinModes()
{
  // setup motor 1 pins
  pinMode(enA, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);

  // setup motor 2 pins
  pinMode(enB, OUTPUT);
  pinMode(in3, OUTPUT);
  pinMode(in4, OUTPUT);
}

// Sets the motors in forward motion mode
void moveMotorsForward()
{
  digitalWrite(in1, LOW);
  digitalWrite(in2, HIGH);
  digitalWrite(in3, LOW);
  digitalWrite(in4, HIGH);
}

// Sets the motors in backward motion mode
void moveMotorsBackward()
{
  digitalWrite(in1, HIGH);
  digitalWrite(in2, LOW);
  digitalWrite(in3, HIGH);
  digitalWrite(in4, LOW);
}

// Setup Headlights pins
void setupHeadlights()
{
  pinMode(HEADLIGHT1, OUTPUT);
  pinMode(HEADLIGHT2, OUTPUT);
}

void headlightsOn()
{
  digitalWrite(HEADLIGHT1, HIGH);
  digitalWrite(HEADLIGHT2, HIGH);
}

void headlightsOff()
{
  digitalWrite(HEADLIGHT1, LOW);
  digitalWrite(HEADLIGHT2, LOW);
}

void setup()
{
  // Setup motors
  setMotorPinModes();
  moveMotorsForward();

  // Setup bluetooth module
  BTSerial.begin(9600);
  Serial.begin(9600);
  BTSerial.print("AT\r\n");

  // setup headlights
  setupHeadlights();
}


void loop()
{
  char buf[5] = {0};

  if (BTSerial.available() >= 4) {
    // Get command
    for(int i = 0; i < 4 ; i++)
      buf[i] = BTSerial.read();
      
    // Set motor power
    if (buf[0] == 'P') {
      // Get commanded speed
      int motorSpeed = atoi(&(buf[1]));
      motorSpeed = map(motorSpeed, 0, 100, -50, 50);

      // Decide forward or backward movement
      if (motorSpeed < 0)
        moveMotorsBackward();
      else
        moveMotorsForward();

      // Get motor power
      power = abs(motorSpeed);

      // Decide each motor speed based on received power and current steer
      if (steer < 0){
        motorBSpeed = power + steer > 0 ? power + steer : 0;
        motorASpeed = power;
      } else {
        motorBSpeed = power;
        motorASpeed = power - steer > 0 ? power - steer : 0;
      }
    }

    // Set each motor speed according to received steer
    if (buf[0] == 'S') {
      // Get commanded steer
      steer = map(atoi(&(buf[1])), 0, 100, -50, 50);

      // Decide each motor speed based on current power and received steer
      if (steer < 0) {
        motorBSpeed = power + steer > 0 ? power + steer : 0;
        motorASpeed = power;
      } else {
        motorBSpeed = power;
        motorASpeed = power - steer > 0 ? power - steer : 0;
      }
        
    }

    // Turn headlights on
    if (buf[0] == 'N') {
      headlightsOn();
    }

    // Turn headlights off
    if (buf[0] == 'F') {
      headlightsOff();
    }

    // Reset arduino receive buffer
    if (buf[0] == 'R') {
      headlightsOn();
      delay(1000);
      headlightsOff();
      while(BTSerial.available()) {
        char a = BTSerial.read();
      }
    }

    // Set each motor's speed
    analogWrite(enA, map(motorASpeed, 0, 50, 0, 255));
    analogWrite(enB, map(motorBSpeed, 0, 50, 0, 255));
  }
}
